@@ -2,15 +2,13 @@ use std::{
     collections::{btree_map, BTreeMap, BTreeSet},
     io,
     net::SocketAddr,
-    sync::{
-        atomic::{AtomicBool, Ordering},
-        Arc, Condvar, Mutex,
-    },
+    sync::{Arc, Condvar, Mutex},
     time::{Duration, Instant},
 };
 
 use junction_api::Hostname;
 use rand::Rng;
+use tokio::sync::Notify;
 
 use crate::load_balancer::EndpointGroup;
 
@@ -35,18 +33,26 @@ pub(crate) struct StdlibResolver {
     inner: Arc<StdlibResolverInner>,
 }
 
-// internal state for a stdlib resolver, shared between all of its workers. the
-// internal ResolverTasks functions as a work queue, and worker threads use a
-// mutex/condvar pair to wait for the next task or to chill until a new task
-// is added to the queue.
 #[derive(Debug)]
 struct StdlibResolverInner {
     lookup_interval: Duration,
     lookup_jitter: Duration,
 
+    // a mutex/condvar pair wrapped around resolver state. see ResolverState for
+    // exactly what that does. nothing here should be much more than locking
+    // around accessing that struct.
     cond: Condvar,
     tasks: Mutex<ResolverState>,
-    shutdown: Arc<AtomicBool>,
+
+    // a notify to let async callers know names have changed while they're
+    // waiting. not specific to any name - will definitely get busy and have
+    // spurious wakeups at hgih volumes of names.
+    async_notify: Notify,
+
+    // the the number of worker threads the resolver was started with. if the
+    // number of references to this struct ever drops, it's time for the
+    // resolver threads to shut down.
+    worker_count: usize,
 }
 
 macro_rules! no_poison {
@@ -57,7 +63,7 @@ macro_rules! no_poison {
 
 impl Drop for StdlibResolver {
     fn drop(&mut self) {
-        self.shutdown();
+        // self.shutdown();
     }
 }
 
@@ -72,7 +78,8 @@ impl StdlibResolver {
             lookup_jitter,
             tasks: Mutex::new(ResolverState::default()),
             cond: Condvar::new(),
-            shutdown: Arc::new(AtomicBool::new(false)),
+            async_notify: Notify::new(),
+            worker_count: threads,
         };
         let resolver = StdlibResolver {
             inner: Arc::new(inner),
@@ -95,18 +102,57 @@ impl StdlibResolver {
         tasks.get_endpoints(hostname, port)
     }
 
-    fn shutdown(&self) {
-        self.inner.shutdown.store(true, Ordering::Release);
-    }
+    pub(crate) async fn get_endpoints_await(
+        &self,
+        hostname: &Hostname,
+        port: u16,
+    ) -> Option<Arc<EndpointGroup>> {
+        // fast path: the endpoints are in the map.
+        if let Some(endpoints) = self.get_endpoints(hostname, port) {
+            return Some(endpoints);
+        }
 
-    fn is_shutdown(&self) -> bool {
-        self.inner.shutdown.load(Ordering::Acquire)
+        // slow path: we're waiting
+        //
+        // on every run through the loop, we need to, IN ORDER, do the
+        // following:
+        //
+        // - register for notifications
+        // - try to get a name in the map, returning it if present
+        // - wait for the next notification
+        //
+        // given this order of events, and the guarantees on Notify, there is no
+        // interleaving of events where we can miss a notification.
+        //
+        // SAFETY: this assumes that the writer half of a notify is always using
+        // notify_waiters and not ever using notify_one.
+        //
+        // for an example that uses notify_one() instead, see Notify
+        //
+        // https://docs.rs/tokio/latest/tokio/sync/futures/struct.Notified.html#method.enable
+        let changed = self.inner.async_notify.notified();
+        tokio::pin!(changed);
+        loop {
+            // check the map
+            if let Some(entry) = self.get_endpoints(hostname, port) {
+                return Some(entry);
+            }
+
+            // wait for a change
+            changed.as_mut().await;
+
+            // this uses Pin::set so we're not allocating/deallocating a new
+            // wakeup future every time.
+            changed.set(self.inner.async_notify.notified());
+        }
     }
 
     pub(crate) fn subscribe(&self, name: Hostname, port: u16) {
         let mut tasks = no_poison!(self.inner.tasks.lock());
-        tasks.pin(name, port);
-        self.inner.cond.notify_all();
+
+        if tasks.pin(name, port) {
+            self.inner.cond.notify_all();
+        }
     }
 
     pub(crate) fn unsubscribe(&self, name: &Hostname, port: u16) {
@@ -125,12 +171,15 @@ impl StdlibResolver {
     }
 
     pub(crate) fn run(&self) {
-        tracing::trace!("thread starting");
+        tracing::trace!("resolver: worker starting");
         loop {
             // grab the next name
-            tracing::trace!("waiting for next name");
             let Some(name) = self.next_name() else {
-                tracing::trace!("thread exiting");
+                tracing::trace!(
+                    worker_count = self.inner.worker_count,
+                    strong_count = Arc::strong_count(&self.inner),
+                    "resolver: worker exiting"
+                );
                 return;
             };
 
@@ -138,7 +187,7 @@ impl StdlibResolver {
             //
             // this always uses 80 and then immediately discards the port. we
             // don't actually care about what the port is here.
-            tracing::trace!(%name, "starting lookup");
+            tracing::trace!(%name, "resolver: starting lookup");
             let addr = (&name[..], 80);
             let answer = std::net::ToSocketAddrs::to_socket_addrs(&addr).map(|answer| {
                 // TODO: we're filtering out every v6 address here. this isn't
@@ -151,12 +200,16 @@ impl StdlibResolver {
             tracing::trace!(
                 %name,
                 ?answer,
-                "saving answer",
+                "resolver: saving answer",
             );
             self.insert_answer(name, Instant::now(), answer);
         }
     }
 
+    fn is_shutdown(&self) -> bool {
+        Arc::strong_count(&self.inner) <= self.inner.worker_count
+    }
+
     fn next_name(&self) -> Option<Hostname> {
         let mut tasks = no_poison!(self.inner.tasks.lock());
 
@@ -198,8 +251,18 @@ impl StdlibResolver {
         resolved_at: Instant,
         answer: io::Result<Vec<SocketAddr>>,
     ) {
-        let mut tasks = no_poison!(self.inner.tasks.lock());
-        tasks.insert_answer(&name, resolved_at, answer);
+        // grab the lock in a tight scope
+        {
+            let mut tasks = no_poison!(self.inner.tasks.lock());
+            tasks.insert_answer(&name, resolved_at, answer);
+        }
+
+        // after the lock is free:
+        //
+        // notify all async callers in get_await. since there's no
+        // blocking get_* method, there are no sync callers to notify.
+        tracing::trace!("notifying waiters");
+        self.inner.async_notify.notify_waiters();
     }
 }
 
@@ -224,7 +287,17 @@ struct NameInfo {
 #[derive(Debug, Default)]
 struct PortInfo {
     pinned: bool,
-    endpoint_group: Arc<EndpointGroup>,
+    endpoint_group: Option<Arc<EndpointGroup>>,
+}
+
+impl PortInfo {
+    fn set_addrs(&mut self, port: u16, addrs: &[SocketAddr]) {
+        let addrs = addrs.iter().cloned().map(|mut addr| {
+            addr.set_port(port);
+            addr
+        });
+        self.endpoint_group = Some(Arc::new(EndpointGroup::from_dns_addrs(addrs)))
+    }
 }
 
 impl NameInfo {
@@ -244,11 +317,7 @@ impl NameInfo {
                 // EndpointGroup for each port.
                 if Some(&addrs) != self.last_addrs.as_ref() {
                     for (port, port_info) in self.ports.iter_mut() {
-                        let addrs = addrs.iter().cloned().map(|mut addr| {
-                            addr.set_port(*port);
-                            addr
-                        });
-                        port_info.endpoint_group = Arc::new(EndpointGroup::from_dns_addrs(addrs))
+                        port_info.set_addrs(*port, &addrs);
                     }
                     self.last_addrs = Some(addrs);
                 }
@@ -300,7 +369,7 @@ impl ResolverState {
     fn get_endpoints(&self, hostname: &Hostname, port: u16) -> Option<Arc<EndpointGroup>> {
         let name_info = self.0.get(hostname)?;
         let port_info = name_info.ports.get(&port)?;
-        Some(port_info.endpoint_group.clone())
+        port_info.endpoint_group.clone()
     }
 
     fn insert_answer(
@@ -319,10 +388,19 @@ impl ResolverState {
         }
     }
 
-    fn pin(&mut self, hostname: Hostname, port: u16) {
-        let name_info = self.0.entry(hostname).or_default();
+    fn pin(&mut self, hostname: Hostname, port: u16) -> bool {
+        let (created, name_info) = match self.0.entry(hostname) {
+            btree_map::Entry::Vacant(entry) => (true, entry.insert(Default::default())),
+            btree_map::Entry::Occupied(entry) => (false, entry.into_mut()),
+        };
         let port_info = name_info.ports.entry(port).or_default();
         port_info.pinned = true;
+
+        if let Some(addrs) = &name_info.last_addrs {
+            port_info.set_addrs(port, addrs);
+        }
+
+        created
     }
 
     fn remove(&mut self, hostname: &Hostname, port: u16) {
@@ -548,6 +626,39 @@ mod test {
         );
     }
 
+    #[test]
+    fn test_pin_new_port() {
+        let mut resolver = ResolverState::default();
+
+        update_all(
+            &mut resolver,
+            [("www.junctionlabs.io", 80), ("www.junctionlabs.io", 443)],
+        );
+
+        resolver.insert_answer(
+            &Hostname::from_static("www.junctionlabs.io"),
+            Instant::now(),
+            // the port here shouldn't matter
+            Ok(vec![SocketAddr::new(IpAddr::V4(Ipv4Addr::LOCALHOST), 1234)]),
+        );
+
+        resolver.pin(Hostname::from_static("www.junctionlabs.io"), 7777);
+
+        let endpoints: Vec<_> = resolver
+            .get_endpoints(&Hostname::from_static("www.junctionlabs.io"), 7777)
+            .unwrap()
+            .iter()
+            .cloned()
+            .collect();
+        assert_eq!(
+            endpoints,
+            vec![crate::EndpointAddress::SocketAddr(SocketAddr::new(
+                IpAddr::V4(Ipv4Addr::LOCALHOST),
+                7777,
+            ))]
+        );
+    }
+
     #[test]
     fn test_reset_drops_inflight() {
         let mut resolver = ResolverState::default();
