@@ -1,7 +1,5 @@
 use crate::EndpointAddress;
-use junction_api::backend::{
-    Backend, LbPolicy, RingHashParams, SessionAffinityHashParam, SessionAffinityHashParamType,
-};
+use junction_api::backend::{Backend, LbPolicy, RequestHashPolicy, RequestHasher, RingHashParams};
 use std::{
     collections::BTreeMap,
     hash::Hash,
@@ -88,12 +86,12 @@ pub enum LoadBalancer {
 }
 
 impl LoadBalancer {
-    pub(crate) fn load_balance<'ep>(
+    pub(crate) fn load_balance<'e>(
         &self,
         url: &crate::Url,
         headers: &http::HeaderMap,
-        locality_endpoints: &'ep EndpointGroup,
-    ) -> Option<&'ep crate::EndpointAddress> {
+        locality_endpoints: &'e EndpointGroup,
+    ) -> Option<&'e crate::EndpointAddress> {
         match self {
             LoadBalancer::RoundRobin(lb) => lb.pick_endpoint(locality_endpoints),
             LoadBalancer::RingHash(lb) => {
@@ -166,7 +164,7 @@ impl RingHashLb {
         &self,
         url: &crate::Url,
         headers: &http::HeaderMap,
-        hash_params: &Vec<SessionAffinityHashParam>,
+        hash_params: &Vec<RequestHashPolicy>,
         endpoint_group: &'e EndpointGroup,
     ) -> Option<&'e EndpointAddress> {
         let request_hash =
@@ -410,20 +408,20 @@ mod test_ring_hash {
 /// - https://github.com/grpc/proposal/blob/master/A42-xds-ring-hash-lb-policy.md#xds-api-fields
 /// - https://github.com/envoyproxy/envoy/blob/main/source/common/http/hash_policy.cc#L236-L257
 pub(crate) fn hash_request(
-    hash_params: &Vec<SessionAffinityHashParam>,
+    hash_policies: &Vec<RequestHashPolicy>,
     url: &crate::Url,
     headers: &http::HeaderMap,
 ) -> Option<u64> {
     let mut hash: Option<u64> = None;
 
-    for hash_param in hash_params {
-        if let Some(new_hash) = hash_target(hash_param, url, headers) {
+    for hash_policy in hash_policies {
+        if let Some(new_hash) = hash_component(hash_policy, url, headers) {
             hash = Some(match hash {
                 Some(hash) => hash.rotate_left(1) ^ new_hash,
                 None => new_hash,
             });
 
-            if hash_param.terminal {
+            if hash_policy.terminal {
                 break;
             }
         }
@@ -432,16 +430,16 @@ pub(crate) fn hash_request(
     hash
 }
 
-fn hash_target(
-    hash_param: &SessionAffinityHashParam,
-    _url: &crate::Url,
+fn hash_component(
+    policy: &RequestHashPolicy,
+    url: &crate::Url,
     headers: &http::HeaderMap,
 ) -> Option<u64> {
-    match &hash_param.matcher {
-        SessionAffinityHashParamType::Header { name } => {
+    match &policy.hasher {
+        RequestHasher::Header { name } => {
             let mut header_values: Vec<_> = headers
                 .get_all(name)
-                .into_iter()
+                .iter()
                 .map(http::HeaderValue::as_bytes)
                 .collect();
 
@@ -453,6 +451,11 @@ fn hash_target(
                 Some(thread_local_xxhash::hash_iter(header_values))
             }
         }
+        RequestHasher::QueryParam { ref name } => url.query().map(|query| {
+            let matching_vals = form_urlencoded::parse(query.as_bytes())
+                .filter_map(|(param, value)| (&param == name).then_some(value));
+            thread_local_xxhash::hash_iter(matching_vals)
+        }),
     }
 }
 
